#
#
# $Id: bbsboard.rb,v 1.24 2004/02/09 14:46:58 Yuukis Exp $
#
#
$KCODE = 'e'

require 'nkf'

def File.copy(from, to)
	f = open(to, 'w')
	f.write IO.readlines(from).join
	f.close
end


class BBSBoard
	S = 'subject.txt'
	NEXT = 'next'
	CACHE = 'cache'
	OLD = 'old'
	PAST = 'past'

	attr_reader :bookmark, :abone, :flags

	def initialize(board, flags = false)
		@server ||= BBSMenu::Servers[board]
		@board ||= board
		@bookmark ||= Bookmark.new(board)
		@abone ||= Abone.new(board)
		@brd_dir ||= CFG::SYS.root + @board
		@locked = false

		if flags
			@flags = flags
		else
			flags ||= Struct.new('SubjectFilterFlags', :bookmark, :abone, :update, :cache_plus)
			@flags ||= flags.new(false, true, false, false)
		end

		Dir.mkdir("#{@brd_dir}") unless File.exist?("#{@brd_dir}")
		Dir.mkdir("#{@brd_dir}/#{NEXT}/") unless File.exist?("#{@brd_dir}/#{NEXT}/")
		Dir.mkdir("#{@brd_dir}/#{CACHE}/") unless File.exist?("#{@brd_dir}/#{CACHE}/")

		clean
	end

	def parse(fn, type)
		subjects = []
		@titleHash = {} if type != 'old'
		n = 1
		re = /(.*)<>(.*)\((\d*)\)/
		IO.readlines("#{@brd_dir}/#{NEXT}/#{fn}").each {|thread|
			thread.sub!(re) {
				d = $1 ; t = $2 ; r = $3
				subjects << [d, r, n]
				if type != 'old'
					@titleHash[n]= String2ch.dat2txt!(t, true)
					n += 1
				end
			}
		}
		return(subjects)
	end
	private :parse

	def get
		read_parsed
	end

	def get_find(word)
		read_parsed(word)
	end

	def mkcache
		results = []
		dregex = /\d*\.dat\.[cache|info|last|post]/
		caches = {}
		oldSHash = {}
		c = Dir.entries("#{@brd_dir}/#{CACHE}/").delete_if {|x| x =~ dregex}
		c.each {|x| caches[x]= 1}
		newSubjects = parse("#{S}", 'new')
		oldSubjects = parse("#{S}.#{OLD}", 'old')
		oldSubjects.each {|k, v, x| oldSHash[k]= v}

		newSubjects.each {|th|
			# th[0]=> dat No., th[1]=> Re: No., th[2]=> title No. in @titleHash
			if caches.include?(th[0])
				th.unshift("C")
			else
				th.unshift(" ")
			end
			if oldSHash.include?(th[1]) == false
				th.unshift("N")
			elsif th[2].to_i > oldSHash[th[1]].to_i
				th.unshift("U")
			else
				th.unshift(" ")
			end
			cfu = ''
			cfu = IO.readlines("#{@brd_dir}/#{CACHE}/#{th[2]}").size.to_s if caches.include?(th[2])
			th[4, 0]= cfu
			th[2, 0]= compare(th[3], cfu)
			th[6] = @titleHash[th[6]]
			results << th
		}
		write_parsed(results)
		return(results)
	end
	private :mkcache

	def compare(dat, cache)
		return(' ') if cache == ''
		return('=') if dat.to_i == cache.to_i
		return('+') if dat.to_i > cache.to_i
		return('?')
	end
	private :compare

	def read_parsed(word = false)
		subjects = []
		array = IO.readlines("#{@brd_dir}/#{S}.#{CACHE}")
		array.each {|line|
			th = line.split('<>')
			th[6].gsub!(/&lt;&gt;/, "<>")
			if word
				subjects << th if th[6] =~ /#{word}/ie
			else
				subjects << th
			end
		}

		# if word
		# 	return(subjects) 
		# else
			return(filter(subjects)) 
		# end
	end
	private :read_parsed

	def write_parsed(array)
		str = ''
		array.each {|th|
			th[6].gsub!(/<>/, "&lt;&gt;")
			str << th.join("<>").concat("\n")
		}
		fh = open("#{@brd_dir}/#{NEXT}/#{S}.#{CACHE}", 'w')
		fh.write str
		fh.close
	end
	private :write_parsed

	def filter(array)

		x = array.clone
		x.reject! {|i| @bookmark.include?(i[3]) == false } if @flags.bookmark
		x.reject! {|i| @abone.include?(i[3]) } if @flags.abone
		if @flags.update
			y = /U|N/
			x.reject! {|i| i[0] !~ y } 
		end
		x.reject! {|i| i[2] != '+' } if @flags.cache_plus
		x
	end
	private :filter

	def cached(dat, board = false)
		board = @board unless board
		str = ''
		dregex = /#{Regexp.quote(dat)}/

		while @locked
			@locked = true if @locked == false
		end

		oldCache = IO.readlines("#{CFG::SYS.root}#{board}/#{S}.#{CACHE}").join
		nCache = open("#{CFG::SYS.root}#{board}/#{S}.#{CACHE}", 'w')

		oldCache.each {|th|
			if th =~ dregex
				a = th.split('<>')
				th = a[0] + '<>C<>=<>' + a[3] + '<>' + \
					a[4] + '<>' + a[4] + '<>' + a[6]
			end
			str << th
		}

		#nCache.flock(LOCK_EX)
		nCache.write str
		nCache.close
		@locked = false
	end

	def finish
		nexts = Dir.glob("#{@brd_dir}/#{NEXT}/*")
		nexts.each {|nfn| File.rename(nfn, "#{@brd_dir}/#{File.basename(nfn)}") }
	end
	private :finish

	def rollback
		nexts = Dir.glob("#{@brd_dir}/#{NEXT}/*")
		nexts.each {|nfn| File.unlink(nfn) }
	end

	alias clean rollback
end


class LiveBBSBoard < BBSBoard
	def get(get = false)
		get = true unless File.exist?("#{@brd_dir}/#{S}")
		if get
			fchk, update = download

			return([false, update]) unless fchk  # return download failed.
			return([false, true]) unless update  # return not update.

			mkcache
			finish
		end
		return([true, super()])
	end

	def download
		if File.exist?("#{@brd_dir}/#{S}")  #  copy ʤȲǾ
			File.copy("#{@brd_dir}/#{S}", "#{@brd_dir}/#{NEXT}/#{S}.#{OLD}")
		end
		result, response = Net2ch.get_keep("http://#{@server}/#{@board}/#{S}", "#{@brd_dir}/#{NEXT}/#{S}")
		return([false, response.body]) unless result

		case response.code
		when '304'
			return([true, false])
		when '200'
			str = NKF.nkf('-exS', IO.readlines("#{@brd_dir}/#{NEXT}/#{S}").join)
			fh = open("#{@brd_dir}/#{NEXT}/#{S}", 'w')
			fh.write str
			fh.close
			unless File.exist?("#{@brd_dir}/#{S}")  # Ĥξ
				File.copy("#{@brd_dir}/#{NEXT}/#{S}", "#{@brd_dir}/#{NEXT}/#{S}.#{OLD}")
			end
			return([true, true])
		end
	end
	private :download
end


class PastBBSBoard < BBSBoard
	def initialize(board, flags = false)
		@bookmark = Bookmark.new(board, "/#{PAST}")
		@abone = Abone.new(board, "/#{PAST}")
		@brd_dir = "#{CFG::SYS.root}#{board}/#{PAST}"
		@live_brd_dir = CFG::SYS.root + board

		super
		clean
	end

	def get(clean = false)
		clean = true unless File.exist?("#{@brd_dir}/#{S}")
		if clean
			mv_old
			mkidx

			mkcache
			finish

			manage_sphandles
		end
		return([true, super()])
	end

	def trash(fn)
		if fn =~ /\d+\.dat$/
			['', '.cache', '.info', '.last', '.post'].each {|suffix|
				if File.exist?("#{@brd_dir}/#{CACHE}/#{fn}#{suffix}")
					Trash.drop("#{@brd_dir}/#{CACHE}/#{fn}#{suffix}")
				end
			}

			str = ''
			IO.readlines("#{@brd_dir}/#{S}.#{CACHE}").join.each {|th|
				str << th if th !~ /#{fn}/
			}

			n = open("#{@brd_dir}/#{S}.#{CACHE}", 'w')
			n.write str
			n.close
		end
	end

	def mkidx
		if File.exist?("#{@brd_dir}/#{S}")  #  copy ʤȲǾ
			File.copy("#{@brd_dir}/#{S}", "#{@brd_dir}/#{NEXT}/#{S}.#{OLD}")
		end

		files = []
		Dir.entries("#{@brd_dir}/#{CACHE}").each {|x| files << x if x =~ /\d+\.dat$/ }
		
		idx = ''
		files.each {|x|
			title = String2ch.get_title("#{@brd_dir}/#{CACHE}/#{x}")
			title.gsub!('>', '&gt;')
			title.gsub!('<', '&lt;')
		
			idx << x
			idx << '<>'
			idx << title
			idx << " (#{IO.readlines("#{@brd_dir}/#{CACHE}/#{x}").size.to_s})"
			idx << "\n"
		}

		fh = open("#{@brd_dir}/#{NEXT}/#{S}", 'w')
		fh.write idx
		fh.close
		unless File.exist?("#{@brd_dir}/#{S}")  # ξ
			File.copy("#{@brd_dir}/#{NEXT}/#{S}", "#{@brd_dir}/#{NEXT}/#{S}.#{OLD}")
		end
	end
	private :mkidx

	def mv_old
		lives = []
		IO.readlines("#{@live_brd_dir}/#{S}").each {|line|
			lives << line.split('<>').shift
		}


		Dir.entries("#{@live_brd_dir}/#{CACHE}").each {|x|
			if x =~ /\d+\.dat$/ and unless lives.index(x)
				['', '.cache', '.info', '.last', '.post'].each {|suffix|
					if File.exist?("#{@live_brd_dir}/#{CACHE}/#{x}#{suffix}")
						File.rename("#{@live_brd_dir}/#{CACHE}/#{x}#{suffix}", "#{@brd_dir}/#{CACHE}/#{x}#{suffix}")
					end
				}
			end ; end
		}
	end
	private :mv_old

	def manage_sphandles
		live_stxt = IO.readlines("#{@live_brd_dir}/#{S}").join
		past_stxt = IO.readlines("#{@brd_dir}/#{S}").join

		live_bookmark = Bookmark.new(@board)
		live_abone = Abone.new(@board)

		[[live_bookmark, @bookmark], [live_abone, @abone]].each {|live, past|
			live.delete('')  # ݸ
			live.each {|dat|
				unless live_stxt.index(dat)
					live.delete(dat)
					past.add(dat) if past_stxt.index(dat)
				end
			}

			past.each {|dat| past.delete(dat) unless past_stxt.index(dat) }
		}
	end
	private :manage_sphandles

end
