#
#
# $Id: gtkext.rb,v 1.15 2003/12/21 03:09:00 Yuukis Exp $
#
#

module Gtk

	class Text
		def add_vikey_events(vadj)
			signal_connect('key_press_event') {|w, e|
				case e.keyval
				when Gdk::GDK_space
					# vadj.set_value(vadj.value + vadj.page_size / 1.1)
					# set signal in Ci.rb.
				when Gdk::GDK_BackSpace
					vadj.set_value(vadj.value - vadj.page_size / 1.1)
				when Gdk::GDK_j
					vadj.set_value(vadj.value + vadj.page_size / 10)
				when Gdk::GDK_k
					vadj.set_value(vadj.value - vadj.page_size / 10)
				when Gdk::GDK_g
					vadj.set_value(vadj.lower)
				when Gdk::GDK_G
					vadj.set_value(vadj.upper)
				end
			}
		end

		def add_mousescroll_events(vadj)
			signal_connect('button_press_event') {|w, e|
				case e.button
				when 5
					vadj.set_value(vadj.value + vadj.page_size / 4)
				when 4
					vadj.set_value(vadj.value - vadj.page_size / 4)
				end
			}
		end

		def adjustment_switch(adj, select_idx)
			Logger.write(__FILE__, __LINE__, "Index => #{select_idx}") if $DEBUG
			Logger.write(__FILE__, __LINE__, "Widget String Size => #{self.get_length}") if $DEBUG

			adj_one = adj.upper / self.get_chars(0, self.get_length).split("\n").size
			adj_old = adj.value
			adj_new = adj_one * self.get_chars(0, select_idx).split("\n").size
			if adj_new < adj_old  # 
				adj.set_value(adj_new) if adj_old - adj_new > adj.page_size
			elsif adj_new > adj_old  # 
				adj.set_value(adj_new) if adj_new - adj_old > adj.page_size
			end
			self.set_position(select_idx)
		end
	end


	class CList
		def add_vikey_events(vadj)
			signal_connect('key_press_event') {|w, e|
				i = vadj.upper / self.rows  # AdjustmentʻCList1Ԥι⤵
				x = 0
				case e.keyval
				when Gdk::GDK_j
					each_selection {|row| x = row + 1 }
					select_row(x, 0)
					if x > (vadj.value + vadj.page_size + i) / i - 1.8  # MagicNumber. XD
						tmp_vadj = vadj.value + i
						if tmp_vadj + vadj.page_size < vadj.upper
							vadj.set_value(tmp_vadj)
						else
							vadj.set_value(vadj.upper - vadj.page_size)
						end
					end
				when Gdk::GDK_k
					each_selection {|row| x = row - 1 }
					select_row(x, 0)
					if x < (vadj.value - i) / i + 0.8  # MagicNumber. XD
						tmp_vadj = vadj.value - vadj.upper / self.rows
						if tmp_vadj + vadj.page_size > vadj.lower
							vadj.set_value(tmp_vadj)
						else
							vadj.set_value(vadj.lower)
						end
					end
				when Gdk::GDK_g
					select_row(0, 0)
					vadj.set_value(vadj.lower)
				when Gdk::GDK_G
					select_row(self.rows - 1, 0)
					vadj.set_value(vadj.upper - vadj.page_size)
				end
			}
		end
	end
end


module GtkExt

	def GtkExt.set_tooltip(widget, text)
		tooltip = Gtk::Tooltips.new
		tooltip.set_tip(widget, text, nil)
	end

	def GtkExt.parent_window(widget)
		if widget.kind_of? Gtk::Window
			return(widget)
		else
			return(parent_window(widget.parent))
		end
	end


	class AdjInfo
		def initialize(board, dat, adj, dir_name = false)
			@board = board
			@dat = dat
			@adj = adj
			if dir_name
				@fn = "#{CFG::SYS.root}#{@board}/#{dir_name}/cache/#{@dat}.info"
			else
				@fn = "#{CFG::SYS.root}#{@board}/cache/#{@dat}.info"
			end
		end

		def get
			if File.exist?(@fn)
				return(IO.readlines(@fn).join.to_f)
			else
				return(0)
			end
		end

		def set
			fh = open(@fn, 'w')
			fh.write @adj.value.to_s
			fh.close
		end

		alias write set

		def exist?
			return(File.exist?(@fn))
		end
	end


	class Pixmap
	    def initialize(w, dir)
			@w = w
			@dir = dir
			@close = [
				"10 7 2 1",
				"  c black",
				".       c None s None",
				".  ....  .",
				"..  ..  ..",
				"...    ...",
				"....  ....",
				"...    ...",
				"..  ..  ..",
				".  ....  ."
			]
			@next = [
				"5 7 2 1",
				"  c black",
				".       c None s None",
				"  ...",
				".  ..",
				"..  .",
				"...  ",
				"..  .",
				".  ..",
				"  ..."
			]
			@prev = [
				"7 7 2 1",
				"  c black",
				".       c None s None",
				"....  .",
				"...  ..",
				"..  ...",
				".  ....",
				"..  ...",
				"...  ..",
				"....  ."
			]
		end
	
		def get(name)
			if name == 'close'
				pix, mask = Gdk::Pixmap.create_from_xpm_d(@w.window, nil, @close)
			elsif name == 'next'
				pix, mask = Gdk::Pixmap.create_from_xpm_d(@w.window, nil, @next)
			elsif name == 'prev'
				pix, mask = Gdk::Pixmap.create_from_xpm_d(@w.window, nil, @prev)
			else
				pix, mask = Gdk::Pixmap.create_from_xpm(@w.window, nil,
					"#{@dir}#{name}.xpm")
			end
			return(Gtk::Pixmap.new(pix, mask))
		end
	end


	class BBSMenuTree < Gtk::ScrolledWindow
		attr_reader :ctree, :vadj

		def initialize(items, tree_column)
			@ctree = Gtk::CTree.new(items, tree_column)
			@ctree.set_line_style(Gtk::CTree::LINES_DOTTED)
			@ctree.set_selection_mode(Gtk::SELECTION_BROWSE)

			@vadj = Gtk::Adjustment.new(0, 0, 0, 0, 0, 0)
			super(nil, @vadj)
			set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_ALWAYS)

			@row = 0
	
			@ctree.signal_connect('select_row') {|w, r, c, e| @row = r }
			@ctree.signal_connect('key_press_event') {|w, e|
				i = @vadj.upper / @ctree.rows  # AdjustmentʻCList1Ԥι⤵
				x = 0
				case e.keyval
				when Gdk::GDK_j
					x = @row + 1  # @rowϾselect_row signalǼ
					@ctree.select_row(x, 0)
					if x > (@vadj.value + @vadj.page_size + i) / i - 1.8  # MagicNumber. XD
						tmpVadj = @vadj.value + i
						if tmpVadj + @vadj.page_size < @vadj.upper
							@vadj.set_value(tmpVadj)
						else
							@vadj.set_value(@vadj.upper - @vadj.page_size)
						end
					end
				when Gdk::GDK_k
					x = @row - 1  # @rowϾselect_row signalǼ
					@ctree.select_row(x, 0)
					if x < (@vadj.value - i) / i + 0.8  # MagicNumber. XD
						tmpVadj = @vadj.value - @vadj.upper / @ctree.rows
						if tmpVadj + @vadj.page_size > @vadj.lower
							@vadj.set_value(tmpVadj)
						else
							@vadj.set_value(@vadj.lower)
						end
					end
				when Gdk::GDK_l
					node = ''
					@ctree.each_selection {|n| node = n }
					@ctree.toggle_expansion(node) unless @ctree.get_node_info(node)[6]
				when Gdk::GDK_g
					@ctree.select_row(0, 0)
					@vadj.set_value(@vadj.lower)
				when Gdk::GDK_G
					@ctree.select_row(@ctree.rows - 1, 0)
					@vadj.set_value(@vadj.upper - @vadj.page_size)
				end
			}
			self.add(@ctree)
		end
		
		def child
			@ctree
		end
	end
end
