#
#
# $Id: gtkpost.rb,v 1.6 2003/12/07 04:18:03 Yuukis Exp $
#
#

class BBSPostBox < Gtk::VBox
	attr_reader :text, :cancel_button, :quote_button, :posting
	POST_POST = '.post'
	PRE_QUOTE_NUM = '>>'
	PRE_QUOTE = '> '
	RE = ' Re: '

	def initialize(board, dat, title)
		@board = board
		@dat = dat
		@title = title
		@getThread = Thread.fork { Thread.exit }
		@update = false
		@posting = false

		super()

		@cancel_button = Gtk::Button.new.add Pix.get('cancel')
		@quote_button = Gtk::Button.new.add Pix.get('quote')
		post_button = Gtk::Button.new.add Pix.get('followup')
			post_button.signal_connect('clicked') {
				@status.set_text(CFG::NLS.pbx_posting)
				@getThread = Thread.fork {
					n = @nameEntry.entry.get_text
					m = @mailEntry.entry.get_text
					@posting = true
					wrote, body = BBSPost.post(@board, @dat, n,  m, @text.get_chars(0, @text.get_length))
					@posting = false
					if wrote
						BBSPost.write_info(n, m, "#{CFG::SYS.root}#{@board}/cache/#{@dat}#{POST_POST}")
						@update = true
						@cancel_button.clicked
					else
						@status.set_text(body)
					end
				}
			}
		GtkExt.set_tooltip(@cancel_button, CFG::NLS.ttp_close)
		GtkExt.set_tooltip(@quote_button, CFG::NLS.ttp_quote)
		GtkExt.set_tooltip(post_button, CFG::NLS.ttp_post)

		aa_button, aa_menu_root, aa_menu = GtkExt::Menus.init(CFG::NLS.ttp_aa, CFG::NLS.pbx_aa)
		BBSPost::AA.get_list.each {|aa|
			label = Gtk::MenuItem.new(aa.chomp)
			aa_menu.append label
			label.signal_connect('activate') { @text.insert_text(aa.chomp, @text.position) }
		}

		filter_button, fi_menu_root, fi_menu = GtkExt::Menus.init(CFG::NLS.ttp_filter,  CFG::NLS.pbx_filter)
		BBSPost::Filter.get_list.each {|fi|
			label = Gtk::MenuItem.new(fi)
			fi_menu.append label
			label.signal_connect('activate') {
				str = @text.get_chars(0, @text.get_length)
				@text.delete_text(0, @text.get_length)
				@text.insert_text(BBSPost::Filter.exec(fi, str), 0)
			}
		}

		blank = Gtk::Label.new('')
		@titleLabel = Gtk::Label.new('')

		sw = Gtk::ScrolledWindow.new
		sw.set_policy(Gtk::POLICY_NEVER, Gtk::POLICY_ALWAYS)
		@text = Gtk::Text.new.set_editable(true)
		@text.signal_connect('key_press_event') {|w, e|
			if e.state == Gdk::CONTROL_MASK
				case e.keyval
				when Gdk::GDK_Return
					post_button.clicked
				when Gdk::GDK_c
					@cancel_button.clicked
				when Gdk::GDK_t
					aa_menu.popup(nil, nil, nil, 1, e.time)
				when Gdk::GDK_f
					fi_menu.popup(nil, nil, nil, 1, e.time)
				end
			end
		}
		sw.add @text

		bBox = Gtk::HBox.new
		bBox.pack_start @titleLabel, false
		bBox.pack_start blank, true
		bBox.pack_start post_button, false
		bBox.pack_start @cancel_button, false
		bBox.pack_start @quote_button, false
		bBox.pack_start aa_button, false
		bBox.pack_start filter_button, false
		
		nameLabel = Gtk::Label.new(' Name ')
		mailLabel = Gtk::Label.new(' Mailto ')
		@nameEntry = Gtk::Combo.new
		@mailEntry = Gtk::Combo.new

		nBox = Gtk::HBox.new
		nBox.pack_start nameLabel, false
		nBox.pack_start @nameEntry
		nBox.pack_start mailLabel, false
		nBox.pack_start @mailEntry

		@status = Gtk::Label.new('')

		self.pack_start bBox, false
		self.pack_start nBox, false
		self.pack_start sw, true
		self.pack_start @status, false
	end

	def post?(p = false)
		@update = false if p
		return(@update)
	end

	def clear
		@text.delete_text(0, @text.get_length)
		[@nameEntry.entry, @mailEntry.entry, @titleLabel, @status].each {|x| x.set_text('') }
	end

	def clear?
		if @titleLabel.get_text == ''
			return(true)
		else
			return(false)
		end
	end

	def show_all(str = false, num = false)
		insert_reply(str, num) if str
		name, mailto = BBSPost.read_info("#{CFG::SYS.root}#{@board}/cache/#{@dat}#{POST_POST}")
		@nameEntry.entry.set_text(name[0])
		@nameEntry.set_popdown_strings(name)
		@mailEntry.entry.set_text(mailto[0])
		@mailEntry.set_popdown_strings(mailto)
		@titleLabel.set_text(RE + @title)
		super()
	end

	def insert_reply(str, num)
		@text.insert nil, nil, nil, PRE_QUOTE_NUM + num.to_s.concat("\n")
		str.each {|line| @text.insert nil, nil, nil, PRE_QUOTE + line.sub(/^\s/, '')}
		@text.insert nil, nil, nil, "\n"
		@text.position = @text.get_length
	end
end
